// GLMain.cpp
// Program entry point and main control is here.
// Richard S. Wright Jr.
// Full Sail, Real World Education

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <gltools.h>
#include <glframe.h>
#include <stopwatch.h>
#include "LawlColors.h"
#include <stdlib.h>
#include "math.h"
#include "scene.h"

int		gWindowWidth	= 800;		// How wide do you want the window?
int		gWindowHeight	= 600;		// How tall do you want the window?
bool	bFullScreen		= false;	// Fullscreen?
bool	bAnimated		= true;		// Do you want the framework to auto-refresh

//Height Map #defines
#define		MAP_SIZE	  1024				// Size Of Our .RAW Height Map (NEW)
#define		STEP_SIZE	  16				// Width And Height Of Each Quad (NEW)
#define		HEIGHT_RATIO  1.5f				// Ratio That The Y Is Scaled According To The X And Z (NEW)
#define		AQHEIGHT	  30
#define		_SIZE	64
#define		TWO_PI 6.2831853071
#define		NUM_FEESH 50
#define		NUM_BUBBLES 200
#define		BUBALG  (-10 + rand()%20) * 0.01f

BYTE g_HeightMap[MAP_SIZE*MAP_SIZE];
BYTE g_WaterMap[MAP_SIZE*MAP_SIZE];// Holds The Height Map Data (NEW)

GLfloat		mesh[_SIZE][_SIZE][3];										// Our Static Mesh
GLfloat		wave_movement = 0.0f;										// Our Variable To Move The Waves Across The Mesh
GLint ShadeWave;
GLint ShadeWaveTime;
GLint FinColor;
GLint FeeshColor;
GLint PhongAlpha;

float scaleValue = 0.025f;					// Scale Value For The Terrain (NEW)
bool bRender = TRUE;						// Polygon Flag Set To TRUE By Default (NEW)
GLhandleARB hShaderProgram = NULL;
GLhandleARB hFishShader = NULL;
GLhandleARB hFinShader = NULL;
GLhandleARB hPhong = NULL;
//int for font list
GLint nFontList;

GLUquadricObj * pObj;

enum RainbowFeesh { F_RED = 0, F_ORANGE, F_YELLOW, F_GREEN, F_BLUE, F_INDIGO, F_VIOLET, F_COLORMAX};

GLFrame		Camera;
CStopWatch	FrameTimer;
struct Spheres
{
	float x;
	float y;
	float z;
};
struct Loc
{
	float x;
	float y;
	float z;
};
struct Feesh
{
	float x;
	float y;
	float z;
	float r;
	GLFrame lolwtf;
	float SwimSpeed;
	int c;
	int s;

};
struct Bubble
{
	float x;
	float y;
	float z;
	float xV;
	float yV;
	float zV;
};

Spheres Stuff[100];
Loc Sand[3];
Loc Glass[3];
Loc Water;
Loc StencilLoc;
Feesh FEESHES[3][NUM_FEESH];
Bubble Bubs[3][NUM_BUBBLES];
float GlassRadius;
GLuint textureHandles[2];
enum        { NUM_CAUSTICS = 32 };
GLuint      CausticHandle[NUM_CAUSTICS];
GLfloat sPlane[4] = { 0.35, 0.21, 0.0, 0.0 };
GLfloat tPlane[4] = { 0.0, 0.21, 0.35, 0.0 };
GLFrame Aquarium_Center;

int Cur_Caust = 0;

float	fAngle			= 0.0f;

//////////////////////////////////////////////////////////////////////////////
void InitContextGL(void);						
void ShutdownContextGL(void);					
void ResizeContextGL(int nWidth, int nHeight);	
void RenderFrame(void);
void UpdateFrame(void);
void DrawGround(void);
void LoadRawFile(LPSTR strName, int nSize, BYTE *pHeightMap);
int Height(BYTE *pHeightMap, int X, int Y);
void SetVertexColor(BYTE *pHeightMap, int x, int y);
void RenderHeightMap(BYTE pHeightMap[], float fTrans);
void RenderWaterMap(BYTE pHeightMap[], float fTrans);
void SetupRC(HDC hDC);
void Swap2Ortho();
void Swap2Perspective();
void DrawAquarium(int nIndex);
void DrawStencilAquarium(int nIndex);
void AdjustWater(float fTime);
void DrawComplexFish();
void DrawSimpleFish(int cIndex);
void LoadLightMap(void);
void DrawBubbles(int nIndex);
bool LoadTextureTGA(const char *szFilename);
void ResetBubs();

bool LoadTexture(const char *szFilename)
{
	BYTE *pBits = NULL;
	int iHeight, iWidth;

	pBits = gltReadBMPBits (szFilename,&iWidth,&iHeight);
	if(pBits == NULL)
	{
		MessageBox(NULL, "Hey dodo, you can't copy files", NULL, MB_OK);
		return false;
	}
	glPixelStorei (GL_UNPACK_ALIGNMENT, 4);
	//glTexImage2D (GL_TEXTURE_2D,0,GL_RGB, iWidth,iHeight,0,GL_BGR,GL_UNSIGNED_BYTE,pBits);
	//build 2d mipmaps
	gluBuild2DMipmaps (GL_TEXTURE_2D,GL_RGB,iWidth,iHeight,GL_BGR,GL_UNSIGNED_BYTE,pBits);
	free(pBits); // i know the read function uses malloc

	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

	return true;
}


//////////////////////////////////////////////////////////////////////////////
// Globals
// These may be shared across modules
HINSTANCE	ghInstance = NULL;		// Application instance
HWND		ghMainWnd = NULL;		// Main Window (window manager)
DEVMODE		gDevMode;

#ifdef UNICODE
const wchar_t szGameName[] = TEXT("Win32 OpenGL");
#else
const char	  szGameName[] = TEXT("Holodeck Aquarium");
#endif

//////////////////////////////////////////////////////////////////////////////
// Forward  and external Declarations
LRESULT CALLBACK MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);


//////////////////////////////////////////////////////////////////////////////
// Program Entry Point
//////////////////////////////////////////////////////////////////////////////
int APIENTRY WinMain(HINSTANCE hInstance,
					 HINSTANCE hPrevInstance,
					 LPSTR     lpCmdLine,
					 int       nCmdShow)
{
	/////////////////////////////////////////////
	// Do any program wide Initialization here
	// Change display settings 
	if(bFullScreen == true)
	{
		gDevMode.dmPelsHeight = gWindowHeight;
		gDevMode.dmPelsWidth = gWindowWidth;
		gDevMode.dmSize = sizeof(DEVMODE);
		gDevMode.dmFields = DM_PELSWIDTH | DM_PELSHEIGHT;
		if(ChangeDisplaySettings(&gDevMode, CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL)
		{
			MessageBox(NULL, TEXT("Cannot change to selected desktop resolution."),
				NULL, MB_OK | MB_ICONSTOP);
			return -1;
		}
	}

	/////////////////////////////////////////////
	// Create Main Window. 
	WNDCLASSEX wcex;
	wcex.cbSize			= sizeof(WNDCLASSEX); 
	wcex.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wcex.lpfnWndProc	= (WNDPROC)MainWndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= NULL;
	wcex.hCursor		= (HCURSOR)LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= NULL; //(HBRUSH)GetStockObject(BLACK_BRUSH);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= TEXT("OpenGLWin32Window"); // Should be unique
	wcex.hIconSm		= NULL;

	if(RegisterClassEx(&wcex) == 0)
		return -1;

	// Select window styles
	UINT uiStyle,uiStyleX;
	if(bFullScreen == true)
	{
		uiStyle = WS_POPUP;
		uiStyleX = WS_EX_TOPMOST;
	}
	else
	{
		uiStyle = WS_OVERLAPPEDWINDOW;
		uiStyleX = NULL;
	}

	// Create the main 3D window
	ghMainWnd = CreateWindowEx(uiStyleX, wcex.lpszClassName, szGameName, uiStyle,
		0, 0, gWindowWidth, gWindowHeight, NULL, NULL, hInstance, NULL);

	if (!ghMainWnd)
		return -1;

	// Make sure window manager stays hidden
	ShowWindow(ghMainWnd, SW_SHOW);
	UpdateWindow(ghMainWnd);
	//SetFocus(ghMainWnd);

	/////////////////////////////////////////////
	// Message Pump - Use the form that goes idle and waits for 
	// messages, not continually running.
	MSG msg;
	while(GetMessage(&msg, NULL, 0, 0)) 
	{
		if(!TranslateMessage(&msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}


	/////////////////////////////////////////////
	// Do any program wide shutdown here
	// Restore Display Settings
	if(bFullScreen == true)
		ChangeDisplaySettings(NULL, 0);

	// Return termination code
	return (int)msg.wParam;
}


/////////////////////////////////////////////////////////////////////
// Main window message procedure. This is the window manager for
// the application
LRESULT CALLBACK MainWndProc(HWND hWnd, UINT message, 
							 WPARAM wParam, LPARAM lParam)
{
	static HDC	hDC = NULL;		// GDI Device context
	static HGLRC hRC = NULL;	// OpenGL Rendering context

	switch (message) 
	{
		// Window is created, the application is starting
	case WM_CREATE:
		{
			hDC = GetDC(hWnd);

			static PIXELFORMATDESCRIPTOR pfd = {
				sizeof(PIXELFORMATDESCRIPTOR),		// Size of struct
				1,									// Version of struct
				PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER, // 
				PFD_TYPE_RGBA,	// Always put this
				32,				// Always put this, size of the color buffer
				0,0,0,0,0,0,0,0,0,0,0,0,0, // You have no control over
				16,					       // Depth of the Z buffer
				8,				// Stencil buffer
				0,0,0,0,0,0 };	// Either obsolete, or we don't care about

				int nPF = ChoosePixelFormat(hDC, &pfd);

				SetPixelFormat(hDC, nPF, &pfd);

				// Create rendering context
				hRC = wglCreateContext(hDC);
				wglMakeCurrent(hDC, hRC);
				SetupRC(hDC);
				InitContextGL();
		}
		break;

		// Do not validate, this is sent repeatedly
	case WM_PAINT:
		UpdateFrame();
		RenderFrame();
		SwapBuffers(hDC);
		if(!bAnimated)
			ValidateRect(hWnd, NULL);
		break;

	case WM_SIZE:
		ResizeContextGL(LOWORD(lParam), HIWORD(lParam));
		break;

		// Window is destroyed, it's the end of the application
	case WM_DESTROY:
		// Cleanup
		ShutdownContextGL();
		wglMakeCurrent(hDC, NULL);
		wglDeleteContext(hRC);
		ReleaseDC(hWnd, hDC);
		PostQuitMessage(0);
		break;

		// Window is either full screen, or not visible
	case WM_ACTIVATE:
		{
			// Only handled when full screen mode
			if(bFullScreen == true)
			{
				WINDOWPLACEMENT wndPlacement;
				wndPlacement.length = sizeof(WINDOWPLACEMENT);
				wndPlacement.flags = WPF_RESTORETOMAXIMIZED;
				wndPlacement.ptMaxPosition.x = 0;
				wndPlacement.ptMaxPosition.y = 0;
				wndPlacement.ptMinPosition.x = 0;
				wndPlacement.ptMinPosition.y = 0;
				wndPlacement.rcNormalPosition.bottom = gDevMode.dmPelsHeight;
				wndPlacement.rcNormalPosition.left = 0;
				wndPlacement.rcNormalPosition.top = 0;
				wndPlacement.rcNormalPosition.right = gDevMode.dmPelsWidth;

				if(LOWORD(wParam) == WA_INACTIVE)
				{
					wndPlacement.showCmd = SW_SHOWMINNOACTIVE;
					SetWindowPlacement(hWnd, &wndPlacement);
					ShowCursor(TRUE);
				}
				else
				{
					wndPlacement.showCmd = SW_RESTORE;
					SetWindowPlacement(hWnd, &wndPlacement);
					ShowCursor(FALSE);
				}
			}
		}
		break;


		// Handle keyboard input
	case WM_CHAR:
		// Close program on ESC key press
		if(wParam == 27)
			DestroyWindow(hWnd);
		break;

	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;

	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Do "one time" initialization here. This function is called imediately after
// the OpenGL Rendering context is created and made current, but before any
// drawing occurs.
void InitContextGL(void)
{
	
	glClearColor(0.0f,0.0f,0.0f, 0.0f);

 	pObj = gluNewQuadric();
 
 	glEnable(GL_DEPTH_TEST);
 	glEnable(GL_CULL_FACE);
 	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations
	LoadLightMap();

	glEnable(GL_POINT_SPRITE_ARB);

	glTexEnvi(GL_POINT_SPRITE, GL_COORD_REPLACE, GL_TRUE);
	glPointParameteri(GL_POINT_SPRITE_COORD_ORIGIN, GL_LOWER_LEFT);

 
 	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 
 	Camera.SetOrigin(0.0f, 2.5f, 7.5f);
	Camera.SetOrigin (0.0f, 5.0f, 40.0f);

  	glEnable(GL_LIGHTING);
  	glEnable(GL_LIGHT0);
  	glLightf(GL_LIGHT0, GL_SPOT_EXPONENT, 180.0f);
  	glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 45.0f);
  	glLightfv(GL_LIGHT0, GL_AMBIENT, fBlack);
  	glLightfv(GL_LIGHT0, GL_DIFFUSE, fWhite);
  	glLightfv(GL_LIGHT0, GL_SPECULAR, fWhite);
  	glLightfv(GL_LIGHT0, GL_POSITION, fOrigin);
  
  	glEnable(GL_LIGHTING);
  	glEnable(GL_LIGHT1);
  	glLightfv(GL_LIGHT1, GL_AMBIENT, fBlack);
  	glLightfv(GL_LIGHT1, GL_DIFFUSE, fDarkGrey);
  	glLightfv(GL_LIGHT1, GL_SPECULAR, fBlue);
  	glLightfv(GL_LIGHT1, GL_POSITION, fOrigin);
  
  	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, fBlack);
	glMateriali(GL_FRONT_AND_BACK, GL_SHININESS, 16);
 
 	
 
 	//FOG
 	/*glEnable(GL_FOG);
 	glFogfv(GL_FOG_COLOR, fLightGrey);
 	glFogf(GL_FOG_START, 2.5f);
 	glFogf(GL_FOG_END, 30.0f);
 	glFogf(GL_FOG_MODE, GL_LINEAR);
 	glHint(GL_FOG_HINT, GL_NICEST);*/
 
 
 	//blending
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
 	//specular 
 	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, fWhite);
 	glMateriali(GL_FRONT_AND_BACK, GL_SHININESS, 128);
 	//Load the hight map file(.raw)
 	LoadRawFile("Batman.raw", MAP_SIZE * MAP_SIZE, g_HeightMap);	// (EG) - This loads our raw file into g_HeightMap.
 	LoadRawFile("water.raw", MAP_SIZE * MAP_SIZE, g_WaterMap);	// (EG) - This loads our raw file into g_WaterMap
 
 	for(int i = 0; i < 100; i++)
 	{
 		Stuff[i].x = (float)(rand() % 15 - 10 / 2.0f);
 		Stuff[i].y = (float)(rand() % 15 - 10 / 2.0f);
 		Stuff[i].z = (float)(rand() % 15 - 10 / 2.0f);
 	}
 	for(int i = 0; i < 3; i++)
 	{
 		Sand[i].x = -11.182f;
 		Sand[i].y = -6.03f;
 		Sand[i].z = -11.182f;
 	}
 	for(int i = 0; i < 3; i++)
 	{
 		Glass[i].x = 12.760f;
 		Glass[i].y = 4.90f;
 		Glass[i].z = 12.76f;
 	}
	Water.x = 2.9f;
	Water.y = 26.5f;
	Water.z = -1.37f;

	StencilLoc.x = 1.60f;
	StencilLoc.y = -1.44f;
	StencilLoc.z = 28.90f;
	
 	//HOLY SHADER CRAP BATMAN
 	//CREATE OUR TESELATED QUAD
 	for (int x = 0; x < _SIZE; x++)
 	{
 		for (int z = 0; z < _SIZE; z++)
 		{
 			mesh[x][z][0] = (float) (_SIZE / 2) - x;						// We Want To Center Our Mesh Around The Origin
 			mesh[x][z][1] = 0.0f;										// Set The Y Values For All Points To 0
 			mesh[x][z][2] = (float) (_SIZE / 2) - z;						// We Want To Center Our Mesh Around The Origin
 		}
 	}

	hShaderProgram = gltLoadShaderPair("Water.vp", "Water.fp");	
	hFishShader = gltLoadShaderPair ("FEESH.vp","FEESH.fp");
 	hFinShader = gltLoadShaderPair ("FIN.vp","FIN.fp");
	hPhong = gltLoadShaderPair ("lux.vp","lux.fp");

	glUseProgramObjectARB(hShaderProgram);
	
	ShadeWave = glGetUniformLocationARB(hShaderProgram, "vWave");
	ShadeWaveTime = glGetUniformLocationARB(hShaderProgram, "fTime");
	FinColor = glGetUniformLocationARB(hFishShader, "lawlColor");
	FeeshColor = glGetUniformLocationARB(hFinShader, "lawlColor");
	PhongAlpha = glGetUniformLocationARB(hPhong,"fAlpha");

	//END SHADER STUFF

	//BEGIN FISH MODEL STUFF

	//END FISH MODEL STUFF
	GlassRadius = 12.8f;

	glEnable(GL_TEXTURE_2D);
	//this loads the mars texture
	glGenTextures(2,textureHandles);
	glActiveTexture(GL_TEXTURE0);
	glBindTexture (GL_TEXTURE_2D, textureHandles[0]);
	if(!LoadTexture("Sand.bmp"))
		return;

	glActiveTexture(GL_TEXTURE0);
	glBindTexture (GL_TEXTURE_2D, textureHandles[1]);
	if(!LoadTexture("Bubble.bmp"))
		return;
	glBindTexture (GL_TEXTURE_2D, textureHandles[0]);


	glTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	float FishRadius = GlassRadius - 5.0f;
	//Init Our Random AQ Location Feesh
	for(int i = 0; i < 3; i++)
	{
		for(int j = 0; j < NUM_FEESH; j++)
		{
			FEESHES[i][j].x = FishRadius;
			FEESHES[i][j].y =  1.5 + rand()%(AQHEIGHT-5);
			FEESHES[i][j].z = -FishRadius + rand()%(int)FishRadius;
			FEESHES[i][j].c = rand()%F_COLORMAX;
			FEESHES[i][j].r = 0;//rand()%361;
			FEESHES[i][j].SwimSpeed = 1 + (rand()%10);
			FEESHES[i][j].lolwtf.SetUpVector(0.0f, 1.0f, 0.0f);
			FEESHES[i][j].lolwtf.SetOrigin(FEESHES[i][j].x,FEESHES[i][j].y,FEESHES[i][j].z);
		}
	}

	for(int i = 0; i < 3; i++)
	{
		for(int j = 0; j<NUM_BUBBLES; j++)
		{
			Bubs[i][j].x = 0;
			Bubs[i][j].y = 0;
			Bubs[i][j].z = 0;
			Bubs[i][j].xV = BUBALG;
			Bubs[i][j].yV = 0.2f + ((rand()%3) * 0.1f);
			Bubs[i][j].zV = BUBALG;
		}
	}

}



//////////////////////////////////////////////////////////////////////////////
// Do any last minute cleanup here. This is called after all rendering, and
// just before the Rendering Context is deleted.
void ShutdownContextGL(void)
{

}	

///////////////////////////////////////////////////////////////////////////////
// This get's called when the window is initialized and whenver/if the window
// changes size. It will get called at least once before the first RenderFrame.
void ResizeContextGL(int nWidth, int nHeight)
{
	glViewport(0, 0, nWidth, nHeight);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(30.0, (double)nWidth/(double)nHeight, 0.1, 500.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}


///////////////////////////////////////////////////////////////////////////////
// Render the next OpenGL frame.
// This gets called whenever the window/screen needs to be updated. It will get
// called repeatedly if the bAnimated flag is set. Otherwise it gets called whenever
// Windows sends a WM_PAINT message, or the programmer calls InvalidateRect.
void RenderFrame(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

	

	static CStopWatch timer;
	static float whaleTime = timer.GetElapsedSeconds ();


	if((timer.GetElapsedSeconds () - whaleTime) > 0.03f)
	{
		if(Cur_Caust<31)
			Cur_Caust++;
		else
			Cur_Caust = 0;
		whaleTime = timer.GetElapsedSeconds ();
	}


	//RENDER DEBUGGING TEXT
// 	glPushMatrix();
// 	{
// 		glDisable(GL_LIGHTING);
// 		Swap2Ortho();
// 		glColor3f(1.0f, 1.0f, 1.0f);
// 		//Scale
// 		glRasterPos2i(0, 60);
// 		glListBase(nFontList);
// 		char szBuffer[128];
// 		sprintf_s(szBuffer,128,"Scale Value: %f", scaleValue);
// 		glCallLists (32, GL_UNSIGNED_BYTE, szBuffer);
// 		//Location
// 		glRasterPos2i(0, 80);
// 		glListBase(nFontList);
// 		sprintf_s(szBuffer,128,"Sand X: %f Y: %f, Z: %f", Sand[0].x, Sand[0].y, Sand[0].z);
// 		glCallLists (64, GL_UNSIGNED_BYTE, szBuffer);
// 
// 		glRasterPos2i(0, 100);
// 		glListBase(nFontList);
// 		sprintf_s(szBuffer,128,"Water X: %f Y: %f, Z: %f", StencilLoc.x, StencilLoc.y, StencilLoc.z);
// 		glCallLists (64, GL_UNSIGNED_BYTE, szBuffer);
// 
// 		glEnable(GL_LIGHTING);
// 
// 		Swap2Perspective();
// 	}
// 	glPopMatrix();

	glPushMatrix();
	{
		Camera.ApplyCameraTransform();

		glActiveTexture(GL_TEXTURE1);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, CausticHandle[Cur_Caust]);
		glActiveTexture(GL_TEXTURE0);

		//Draw Ground and Ceiling(possibly walls as well)
		//do all of this before the stencil test because we want to render all of these things no matter what.
		
		DrawGround();
		glPushMatrix ();
		{
			glTranslatef(0.0f,100.0f,0.0f);
			DrawGround ();
		}
		glPopMatrix();
		glPushMatrix();
		{
			glRotatef (-90, 1.0f, 0.0f, 0.0f);
			glTranslatef(0.0f, 50.0f, 50.0f);		
			DrawGround ();
		}
		glPopMatrix ();
		glPushMatrix ();
		{
			glRotatef(-90, 1.0f, 0.0f, 0.0f);
			glTranslatef (0.0f,-50.0f, 50.0f);
			DrawGround ();
		}
		glPopMatrix ();
		glPushMatrix ();
		{
			glRotatef(-90,1.0f,0.0f,0.0f);
			glRotatef(-90,0.0f,0.0f,1.0f);
			glTranslatef(0.0f, 50.0f, 50.0f);
			DrawGround();
		}
		glPopMatrix ();
		glPushMatrix ();
		{
			glRotatef(-90,1.0f,0.0f,0.0f);
			glRotatef(-90,0.0f,0.0f,1.0f);
			glTranslatef(0.0f,-50.0f,50.0f);
			DrawGround ();
		}
		glPopMatrix ();
		glTranslatef(0.0f, 2.5f, 0.0f);
		
		
		//MAKE OUR WATER STENCIL
		glPushMatrix ();
		{
			DrawStencilAquarium (0);
		}
		glPopMatrix();


		
		
		


		glPushMatrix();
		{
			glPushMatrix();
			{
				glEnable(GL_STENCIL_TEST);
				//glDisable (GL_STENCIL_TEST);
				glTranslatef (Water.x,Water.y,Water.z);
				glRotatef(-90,0.0f,1.0f,0.0f);
				//glScalef(scaleValue, scaleValue, scaleValue);
				//glEnable (GL_TEXTURE_2D);
				glDisable (GL_CULL_FACE);

				//glBindTexture (GL_TEXTURE_2D, textureHandles[0]);
				glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE, fWhite);

				RenderWaterMap (g_WaterMap,1.0f);
				glDisable(GL_STENCIL_TEST);

				glEnable (GL_CULL_FACE);
			}
			glPopMatrix ();
			DrawAquarium (0);
		}
		glPopMatrix();
		
		

		//glFlush();
		glDisable (GL_STENCIL_TEST);
	}
	glPopMatrix();

}


void UpdateFrame(void)
{																			  
	float fDeltaTime = FrameTimer.GetElapsedSeconds();
	FrameTimer.Reset();
	fAngle += 45.0f * fDeltaTime;

    float fRotAngle =  1.0f * fDeltaTime;
    float fTranslate = 5.0f * fDeltaTime;
    
    if (GetAsyncKeyState('W'))
        Camera.MoveForward(fTranslate);
    if (GetAsyncKeyState('S'))
        Camera.MoveForward(-fTranslate);
    if (GetAsyncKeyState('Q'))
        Camera.MoveRight(fTranslate);
    if (GetAsyncKeyState('E'))
        Camera.MoveRight(-fTranslate);
    if (GetAsyncKeyState('A'))
        Camera.RotateLocalY(fRotAngle);
    if (GetAsyncKeyState('D'))
        Camera.RotateLocalY(-fRotAngle);
	if (GetAsyncKeyState(VK_UP))
		Camera.MoveUp(fTranslate);
	if (GetAsyncKeyState(VK_DOWN))
		Camera.MoveUp(-fTranslate);
	if (GetAsyncKeyState('F'))
		Camera.RotateLocalX(fRotAngle);
	if (GetAsyncKeyState('R'))
		Camera.RotateLocalX(-fRotAngle);
	if (GetAsyncKeyState(VK_LEFT))
		Camera.RotateLocalZ(-fRotAngle);
	if (GetAsyncKeyState(VK_RIGHT))
		Camera.RotateLocalZ(fRotAngle);
	/*if (GetAsyncKeyState(VK_ADD))
		scaleValue += 0.001f;
	if (GetAsyncKeyState(VK_SUBTRACT))
		scaleValue -= 0.001f;
	if (GetAsyncKeyState(VK_NUMPAD8))
		StencilLoc.z += 0.01f;
	if (GetAsyncKeyState(VK_NUMPAD2))
		StencilLoc.z -= 0.01f;
	if (GetAsyncKeyState(VK_NUMPAD4))
		StencilLoc.x += 0.01f;
	if (GetAsyncKeyState(VK_NUMPAD6))
		StencilLoc.x -= 0.01f;
	if (GetAsyncKeyState(VK_NUMPAD9))
		StencilLoc.y += 0.01f;
	if (GetAsyncKeyState(VK_NUMPAD3))
		StencilLoc.y -= 0.01f;
	if (GetAsyncKeyState(VK_NUMPAD7))
		GlassRadius += 0.1f;
	if (GetAsyncKeyState(VK_NUMPAD1))
		GlassRadius -= 0.1f;
	if(GetAsyncKeyState (VK_SPACE))
		ResetBubs ();*/
}

void DrawGround(void)
{
	static const float fGroundSize = 50.0f;
	static const float fTileSize   =  1.0f;
	static const float fHeight     =  0.0f;
	glDisable(GL_LIGHTING);
    glLineWidth(2.0f);
	glBegin(GL_LINES);
	{
	    glColor3f(1.0f, 1.0f, 1.0f);
	    glVertex3f(0, fHeight,  fGroundSize);
	    glVertex3f(0, fHeight, -fGroundSize);
	    glVertex3f( fGroundSize, fHeight, 0);
	    glVertex3f(-fGroundSize, fHeight, 0);
	}
	glEnd();	
	
	glLineWidth(1.0f);
	glBegin(GL_LINES);
	{
	    glColor4fv(fGreen);
	    for (float f = -fGroundSize; f <= fGroundSize; f+= fTileSize)
	    {
    	    glVertex3f( f, fHeight,  fGroundSize);   
    	    glVertex3f( f, fHeight, -fGroundSize);   
    	    glVertex3f(  fGroundSize, fHeight, f);   
    	    glVertex3f( -fGroundSize, fHeight, f);   
	    }	
	}
	glEnd();
	glEnable(GL_LIGHTING);
}

// Loads The .RAW File And Stores It In pHeightMap
void LoadRawFile(LPSTR strName, int nSize, BYTE *pHeightMap)
{
	FILE *pFile = NULL;

	// Open The File In Read / Binary Mode.
	pFile = fopen( strName, "rb" );

	// Check To See If We Found The File And Could Open It
	if ( pFile == NULL )	
	{
		// Display Error Message And Stop The Function
		MessageBox(NULL, "Can't Find The Height Map!", "Error", MB_OK);
		return;
	}

	fread( pHeightMap, 1, nSize, pFile );

	// After We Read The Data, It's A Good Idea To Check If Everything Read Fine
	int result = ferror( pFile );

	// Check If We Received An Error
	if (result)
	{
		MessageBox(NULL, "Failed To Get Data!", "Error", MB_OK);
	}

	// Close The File.
	fclose(pFile);
}

int Height(BYTE *pHeightMap, int X, int Y)				// This Returns The Height From A Height Map Index
{
	int x = X % MAP_SIZE;								// Error Check Our x Value
	int y = Y % MAP_SIZE;								// Error Check Our y Value

	if(!pHeightMap) return 0;							// Make Sure Our Data Is Valid

	return pHeightMap[x + (y * MAP_SIZE)];				// Index Into Our Height Array And Return The Height
}

void SetHeight(BYTE *pHeightMap, int X, int Y, int NewY)
{
	int x = X % MAP_SIZE;
	int y = Y % MAP_SIZE;

	pHeightMap[x + (y * MAP_SIZE)] = NewY;
}

void SetVertexColor(BYTE *pHeightMap, int x, int y, float fTrans)		// Sets The Color Value For A Particular Index, Depending On The Height Index
{
	if(!pHeightMap) return;								// Make Sure Our Height Data Is Valid

	float fColor = -0.15f + (Height(pHeightMap, x, y ) / 256.0f);

	// Assign This Blue Shade To The Current Vertex
	glColor4f(fColor, fColor, fColor, fTrans );
}

void RenderHeightMap(BYTE pHeightMap[], float fTrans)					// This Renders The Height Map As Quads
{
	glDisable(GL_LIGHTING);
	glActiveTexture(GL_TEXTURE0);
	glBindTexture (GL_TEXTURE_2D,textureHandles[0]);
	int X = 0, Y = 0;									// Create Some Variables To Walk The Array With.
	int x, y, z;										// Create Some Variables For Readability

	if(!pHeightMap) return;								// Make Sure Our Height Data Is Valid

	

	if(bRender)											// What We Want To Render
		glBegin( GL_QUADS );							// Render Polygons
	else 
		glBegin( GL_LINES );							// Render Lines Instead

	float UVSCALE = 1.0f;
	for ( X = 0; X < (MAP_SIZE-STEP_SIZE); X += STEP_SIZE )
		for ( Y = 0; Y < (MAP_SIZE-STEP_SIZE); Y += STEP_SIZE )
		{
			// Get The (X, Y, Z) Value For The Bottom Left Vertex
			x = X;							
			y = Height(pHeightMap, X, Y );	
			z = Y;							

			// Set The Color Value Of The Current Vertex
			SetVertexColor(pHeightMap, x, z, fTrans);
			glTexCoord2f(0.0f,0.0f);
			glVertex3i(x, y, z);						// Send This Vertex To OpenGL To Be Rendered (Integer Points Are Faster)

			// Get The (X, Y, Z) Value For The Top Left Vertex
			x = X;										
			y = Height(pHeightMap, X, Y + STEP_SIZE );  
			z = Y + STEP_SIZE ;							
			
			// Set The Color Value Of The Current Vertex
			SetVertexColor(pHeightMap, x, z, fTrans);

			glTexCoord2f(0.0f,UVSCALE);
			glVertex3i(x, y, z);						// Send This Vertex To OpenGL To Be Rendered

			// Get The (X, Y, Z) Value For The Top Right Vertex
			x = X + STEP_SIZE; 
			y = Height(pHeightMap, X + STEP_SIZE, Y + STEP_SIZE ); 
			z = Y + STEP_SIZE ;

			// Set The Color Value Of The Current Vertex
			SetVertexColor(pHeightMap, x, z, fTrans);
			
			glTexCoord2f(UVSCALE,UVSCALE);
			glVertex3i(x, y, z);						// Send This Vertex To OpenGL To Be Rendered

			// Get The (X, Y, Z) Value For The Bottom Right Vertex
			x = X + STEP_SIZE; 
			y = Height(pHeightMap, X + STEP_SIZE, Y ); 
			z = Y;

			// Set The Color Value Of The Current Vertex
			SetVertexColor(pHeightMap, x, z,fTrans);

			glTexCoord2f(UVSCALE,0.0f);
			glVertex3i(x, y, z);						// Send This Vertex To OpenGL To Be Rendered
		}
	glEnd();

	glEnable(GL_LIGHTING);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);					// Reset The Color
}

//Botched attempt at producing a sin wave modified height map.
void RenderWaterMap(BYTE pHeightMap[], float fTrans)					// This Renders The Height Map As Quads
{
	static CStopWatch FrameTimer;
	float fTime = FrameTimer.GetElapsedSeconds();
	//FrameTimer.Reset();
	glUseProgramObjectARB(hShaderProgram);

	glUniform3f(ShadeWave,wave_movement, 1.0f, 1.0f);
	glUniform1f(ShadeWaveTime,fTime);
	for (int x = 0; x < _SIZE - 1; x++)
	{
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		//glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, fGreen);
		glBegin(GL_TRIANGLE_STRIP);
		for (int z = 0; z < _SIZE - 1; z++)
		{
			// Set The Wave Parameter Of Our Shader To The Incremented Wave Value From Our Main Program
			//cgGLSetParameter3f(wave, wave_movement, 1.0f, 1.0f);
			
			glVertex3f(mesh[x][z][0], mesh[x][z][1], mesh[x][z][2]);		// Draw Vertex
			glVertex3f(mesh[x+1][z][0], mesh[x+1][z][1], mesh[x+1][z][2]);	// Draw Vertex
			wave_movement += 0.00001f;//(TWO_PI * (2.0f * fTime));
			//fTime = FrameTimer.GetElapsedSeconds();
			//FrameTimer.Reset();//0.00001f;						// Increment Our Wave Movement
			if (wave_movement > TWO_PI)									// Prevent Crashing
				wave_movement = 0.0f;
		}
	
		glEnd();
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		


	}
	glUseProgramObjectARB(NULL);
}
void SetupRC(HDC hDC)
	{
	// Setup the Font characteristics
	HFONT hFont;
	LOGFONT logfont;

	logfont.lfHeight = -20;
	logfont.lfWidth = 0;
	logfont.lfEscapement = 0;
	logfont.lfOrientation = 0;
	logfont.lfWeight = FW_BOLD;
	logfont.lfItalic = FALSE;
	logfont.lfUnderline = FALSE;
	logfont.lfStrikeOut = FALSE;
	logfont.lfCharSet = ANSI_CHARSET;
	logfont.lfOutPrecision = OUT_DEFAULT_PRECIS;
	logfont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	logfont.lfQuality = DEFAULT_QUALITY;
	logfont.lfPitchAndFamily = DEFAULT_PITCH;
	strcpy(logfont.lfFaceName,"Arial");

	// Create the font and display list
	hFont = CreateFontIndirect(&logfont);
	SelectObject (hDC, hFont); 

	
	//Create display lists for glyphs 0 through 128
	nFontList = glGenLists(128);
	wglUseFontBitmaps(hDC, 0, 128, nFontList);

	DeleteObject(hFont);		// Don't need original font anymore

	// Black Background
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f );
	}

void Swap2Ortho()
{
	// Set View port to window dimensions
	glViewport(0, 0, gWindowWidth, gWindowHeight);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluOrtho2D(0,gWindowWidth, gWindowHeight, 0);

	// Viewing transformation
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void Swap2Perspective()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(30.0, (double)gWindowWidth/(double)gWindowHeight, 0.1, 500.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void DrawAquarium (int nIndex)
{
	static float yRot = 0;
	static CStopWatch physicsTimer;
	float fTime = physicsTimer.GetElapsedSeconds();
	yRot += fTime * 10.0f;

	physicsTimer.Reset();

	glTranslatef(Sand[nIndex].x, Sand[nIndex].y, Sand[nIndex].z);

	//Draw the aquarium cylinder
	//DRAW THE BASE OF THE AQUARIUM FIRST
	glUseProgramObjectARB(NULL);
	glActiveTexture(GL_TEXTURE1);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE0);

	glDisable(GL_STENCIL_TEST);
	glPushMatrix ();
	{
		glTranslatef (Glass[nIndex].x,Glass[nIndex].y,Glass[nIndex].z);
		glTranslatef(0.0f,-1.5f,0.0f);
		glRotatef(90,1.0f,0.0f,0.0f);
		gluDisk (pObj,0.0f,GlassRadius + 100,30,30);

	}
	glPopMatrix ();
	glPushMatrix();
	{
		glUniform1f(PhongAlpha,1.0f);
		glUseProgramObjectARB(NULL);

		glDisable (GL_CULL_FACE);
		glTranslatef (Glass[nIndex].x, Glass[nIndex].y-0.01f, Glass[nIndex].z);
		Aquarium_Center.SetOrigin (Glass[nIndex].x, Glass[nIndex].y-0.01f, Glass[nIndex].z);
		glRotatef (-90, 1.0f, 0.0f, 0.0f);
		glMaterialfv (GL_FRONT_AND_BACK,GL_DIFFUSE,fBlack);
		glMaterialfv (GL_FRONT_AND_BACK,GL_SPECULAR,fDarkGrey);
		gluCylinder (pObj,GlassRadius-1, GlassRadius-0.2f,1,30,30);
		glTranslatef(0.0f,0.0f,1.0f);
		gluDisk (pObj,GlassRadius-0.2f,GlassRadius+0.5f,30,10);
		glTranslatef (0.0f,0.0f,-5.0f);
		gluCylinder (pObj,GlassRadius+0.5f,GlassRadius+0.5f,5,30,30);
		glEnable (GL_CULL_FACE);

		glUseProgramObjectARB(NULL);
	}
	glPopMatrix();
	//Draw the back face first

	glPushMatrix ();
	{
		glTranslatef (Glass[nIndex].x, Glass[nIndex].y, Glass[nIndex].z);
		glRotatef (-90,1.0f,0.0f,0.0f);
		glCullFace (GL_FRONT);
		glDisable (GL_TEXTURE_2D);
		glDisable (GL_ALPHA_TEST);
		glEnable (GL_BLEND);
		glMaterialfv (GL_FRONT_AND_BACK,GL_SPECULAR,fWhite);
		glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE, fSTBlue);
		gluCylinder (pObj,GlassRadius,GlassRadius,AQHEIGHT,30,30);
		
	}
	glPopMatrix ();

	

	glPushMatrix();
	{
		glTranslatef (Glass[nIndex].x, Glass[nIndex].y, Glass[nIndex].z);
		glDisable(GL_STENCIL_TEST);
		glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE, fWhite);
		for(int i = 0; i < NUM_FEESH; i++)
		{
 			glPushMatrix ();
 			{
				FEESHES[nIndex][i].r += fTime * (10.0f * FEESHES[nIndex][i].SwimSpeed);
				glRotatef (FEESHES[nIndex][i].r,0.0f,1.0,0.0f);
 				{
 					float nOrigin[3];
 					float nY[3];
 					Aquarium_Center.GetOrigin(nOrigin);
 					FEESHES[nIndex][i].lolwtf.GetUpVector (nY);
 					float newVec[3];
 					newVec[0]= nOrigin[0] - nY[0];
 					newVec[1]= nOrigin[1] - nY[1];
 					newVec[2]= nOrigin[2] - nY[2];
					m3dNormalizeVector(newVec);
					newVec[0] *= -1.0f;
					newVec[1] *= -1.0f;
					newVec[2] *= -1.0f;

					m3dCrossProduct(newVec, nY, newVec);
 					
 					FEESHES[nIndex][i].lolwtf.SetForwardVector (newVec);
 					FEESHES[nIndex][i].lolwtf.ApplyActorTransform ();
					FEESHES[nIndex][i].lolwtf.Normalize ();
 				}
 				
 				//glTranslatef (FEESHES[nIndex][i].x,FEESHES[nIndex][i].y,FEESHES[nIndex][i].z);
 				
				
				//glRotatef (FEESHES[nIndex][i].r,0.0f,1.0f,0.0f);


				DrawSimpleFish(FEESHES[nIndex][i].c);
			}
			glPopMatrix ();
		}
	}
	glPopMatrix ();	

	glPushMatrix ();
	{
		glActiveTexture(GL_TEXTURE1);
		glEnable(GL_TEXTURE_2D);
		glActiveTexture(GL_TEXTURE0);
		glScalef(scaleValue, scaleValue, scaleValue);
		glEnable (GL_TEXTURE_2D);
		glDisable (GL_CULL_FACE);
		glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE, fWhite);
		glBindTexture (GL_TEXTURE_2D, textureHandles[0]);
		RenderHeightMap(g_HeightMap,1.0f);
		glEnable (GL_CULL_FACE);
		glActiveTexture(GL_TEXTURE1);
		glDisable(GL_TEXTURE_2D);
		glActiveTexture(GL_TEXTURE0);
	}
	glPopMatrix ();

	glPushMatrix ();
	{
		glTranslatef (Glass[nIndex].x, Glass[nIndex].y, Glass[nIndex].z);
		DrawBubbles (nIndex);
	}
	glPopMatrix ();

	glActiveTexture(GL_TEXTURE1);
	glDisable (GL_TEXTURE_2D);
	glPushMatrix ();
	{
		glUniform1f(PhongAlpha,0.2f);
		glUseProgramObjectARB(NULL);

		glTranslatef (Glass[nIndex].x, Glass[nIndex].y, Glass[nIndex].z);
		glRotatef (-90,1.0f,0.0f,0.0f);
		glCullFace (GL_BACK);
		glDisable (GL_TEXTURE_2D);
		glDisable (GL_ALPHA_TEST);
		glEnable (GL_BLEND);
		glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE, fSTBlue);
		gluCylinder (pObj,GlassRadius,GlassRadius,AQHEIGHT,30,30);
		glUseProgramObjectARB(NULL);

	}
	glPopMatrix ();
	glEnable(GL_STENCIL_TEST);

}
void DrawStencilAquarium (int nIndex)
{
///////////////////////////////////////////////////////////////////
	glClearStencil (0.0f);
	glEnable(GL_STENCIL_TEST);
	//glClear(GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	///////////////////////////////////////////////////////////////

	glStencilFunc(GL_NEVER, 0x0, 0x0);
	glStencilOp(GL_INCR, GL_INCR, GL_INCR);

	

	//Draw the aquarium cylinder
	//DRAW THE BASE OF THE AQUARIUM FIRST
	glUseProgramObjectARB(NULL);
	//Draw the back face first

	glPushMatrix ();
	{
		glTranslatef(Sand[0].x, Sand[0].y, Sand[0].z);

		glTranslatef (Glass[nIndex].x, Glass[nIndex].y, Glass[nIndex].z);
		glRotatef (-90,1.0f,0.0f,0.0f);
		glCullFace (GL_FRONT);
		glDisable (GL_TEXTURE_2D);
		glDisable (GL_ALPHA_TEST);
		glEnable (GL_BLEND);
		glMaterialfv (GL_FRONT_AND_BACK,GL_SPECULAR,fWhite);
		glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE, fSTBlue);
		gluCylinder (pObj,GlassRadius,GlassRadius,AQHEIGHT,30,30);
	}
	glPopMatrix ();



  	glPushMatrix();
  	{
		glRotatef(-90,1.0f, 0.0f, 0.0f);
		glTranslatef(StencilLoc.x, StencilLoc.y, StencilLoc.z);
 		//glDisable(GL_STENCIL_TEST);
		glDisable(GL_CULL_FACE);
 		//glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE, fWhite);
  		//glTranslatef(Water.x, Water.y+1, Water.z); 
  		gluDisk(pObj,0.0f,GlassRadius,30,15);
		glEnable(GL_CULL_FACE);
 		glEnable(GL_STENCIL_TEST);
  	}
  	glPopMatrix ();
 	

// 	glPushMatrix ();
// 	{
// 		glTranslatef(Sand[0].x, Sand[0].y, Sand[0].z);
// 
// 		glTranslatef (Glass[nIndex].x, Glass[nIndex].y, Glass[nIndex].z);
// 		glRotatef (-90,1.0f,0.0f,0.0f);
// 		glCullFace (GL_BACK);
// 		glDisable (GL_TEXTURE_2D);
// 		glDisable (GL_ALPHA_TEST);
// 		glEnable (GL_BLEND);
// 		glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE, fSTBlue);
// 		gluCylinder (pObj,GlassRadius,GlassRadius,AQHEIGHT,30,30);
// 	}
// 	glPopMatrix ();
	///////////////////////////////////////////////////////////////////////////////////
	glStencilFunc(GL_EQUAL, 0x1, 0x1);
	glStencilOp(GL_DECR, GL_DECR, GL_DECR);
	///////////////////////////////////////////////////////////////////////////////////

}

void AdjustWater(float fTime)
{
	
}

void DrawComplexFish()
{
	//Draw all 10 stupid objects of the ONE fish...
	glPushMatrix();
	{
		glEnableClientState (GL_VERTEX_ARRAY);
		//glEnableClientState (GL_NORMAL_ARRAY);
		//glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glVertexPointer (3,GL_FLOAT,0,verticies_0);
		glNormalPointer(GL_FLOAT,0,normals_0);
		glTexCoordPointer (3,GL_FLOAT,0,textures_0);
		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, fWhite);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, fWhite);
		glColor4fv (fWhite);			
		glDrawElements (GL_TRIANGLES,(MAX_FACE_INDICES_0*3),GL_UNSIGNED_INT,face_indicies_0);
		glDisableClientState(GL_VERTEX_ARRAY);
		glDisableClientState (GL_NORMAL_ARRAY);
		glDisableClientState (GL_TEXTURE_COORD_ARRAY);
	}
	glPopMatrix ();
}
void DrawSimpleFish(int cIndex)
{
	//Draw the uber simple 2 object fish and apply a shader to make it... fishy...
	glPushMatrix();
	{
		
		//Draw a sphere for the feesh body.
		//LOLOLOLWTF?!?!?!!?!?!?!?!?!?!?!?!
		glUseProgramObjectARB(NULL);
		glUseProgramObjectARB(hFishShader);
		glCullFace (GL_BACK);
		switch(cIndex)
		{
		case F_RED:
			glUniform4fv(FeeshColor,1.0f,fRed);
			break;
		case F_ORANGE:
			glUniform4fv(FeeshColor,1.0f,fOrange);
			break;
		case F_YELLOW:
			glUniform4fv(FeeshColor,1.0f,fYellow);
			break;
		case F_GREEN:
			glUniform4fv(FeeshColor,1.0f,fGreen);
			break;
		case F_BLUE:
			glUniform4fv(FeeshColor,1.0f,fBlue);
			break;
		case F_INDIGO:
			glUniform4fv(FeeshColor,1.0f,fCyan);
			break;
		case F_VIOLET:
			glUniform4fv(FeeshColor,1.0f,fMagenta);
			break;

		}

		glDisable(GL_BLEND);
		//glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, fRed);
		gltDrawSphere(1.0f,6,3);
		//Draw The Eye
		glPushMatrix ();
		{
			glUseProgramObjectARB(hFishShader);
			glUniform4fv(FeeshColor,1.0f,fBlack);
			glTranslatef(-0.3f,0.0f,0.0f);
			glTranslatef(0.0f,0.25f,0.0f);
			glScalef(0.5f,0.5f,1.5f);
			gltDrawSphere (0.5f,30,15);
		}
		glPopMatrix ();
		//Draw the cone for the tail of the feesh.
		glTranslatef (1.0f,0.0f,0.0f);
		glRotatef(-90,0.0f,1.0f,0.0f);
		//glRotatef(-90,0.0f,0.0f,1.0f);
		glDisable(GL_CULL_FACE);
		glUseProgramObjectARB(hFinShader);
		glUniform4fv(FinColor,1.0f,fOrange);
		switch(cIndex)
		{
		case F_RED:
			glUniform4fv(FinColor,1.0f,fCyan);
			break;
		case F_ORANGE:
			glUniform4fv(FinColor,1.0f,fRed);
			break;
		case F_YELLOW:
			glUniform4fv(FinColor,1.0f,fWhite);
			break;
		case F_GREEN:
			glUniform4fv(FinColor,1.0f,fDarkGrey);
			break;
		case F_BLUE:
			glUniform4fv(FinColor,1.0f,fRed);
			break;
		case F_INDIGO:
			glUniform4fv(FinColor,1.0f,fYellow);
			break;
		case F_VIOLET:
			glUniform4fv(FinColor,1.0f,fBlack);
			break;

		}


		gluCylinder (pObj,1.0f,0.3f,0.5f,4,2);
		glEnable(GL_CULL_FACE);
		glUseProgramObjectARB(NULL);
		glEnable(GL_BLEND);

	}
	glPopMatrix();	
}

void LoadLightMap(void)
{
	GLbyte *pBits = NULL;
	int iHeight, iWidth;

	GLint components;
	GLenum format;

	glDeleteTextures(NUM_CAUSTICS, CausticHandle);
	glGenTextures(NUM_CAUSTICS, CausticHandle);
	char szBuffer[128];
	// TODO:  Load the Caustic Textures.
	for(int i = 0; i < NUM_CAUSTICS; i++)
	{
		glBindTexture (GL_TEXTURE_2D, CausticHandle[i]);
		if(i < 10)
		{
			sprintf_s (szBuffer,128,"caust0%i.tga",i);
		}
		else
			sprintf_s(szBuffer,128,"caust%i.tga",i);

		pBits = gltLoadTGA  (szBuffer,&iWidth,&iHeight,&components,&format);
		if(pBits == NULL)
		{
			MessageBox(NULL, "Hey dodo, you can't copy files", NULL, MB_OK);
			return;
		}
		glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
		//glTexImage2D (GL_TEXTURE_2D,0,GL_RGB, iWidth,iHeight,0,GL_BGR,GL_UNSIGNED_BYTE,pBits);
		//build 2d mipmaps
		gluBuild2DMipmaps (GL_TEXTURE_2D,components,iWidth,iHeight,format,GL_UNSIGNED_BYTE,pBits);
		free(pBits); // i know the read function uses malloc



		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	}
	glActiveTexture(GL_TEXTURE1);

	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGenfv(GL_S, GL_OBJECT_PLANE, sPlane);
	glTexGenfv(GL_T, GL_OBJECT_PLANE, tPlane);
	glActiveTexture(GL_TEXTURE0);

}

void DrawBubbles(int nIndex)
{
	glUseProgramObjectARB(NULL);
	glDisable(GL_STENCIL_TEST);
	glDisable(GL_BLEND);
	static CStopWatch timer;
	static float bubTime = timer.GetElapsedSeconds ();
	static int BubbleToDraw = 0;



	if((timer.GetElapsedSeconds () - bubTime) > 0.03f)
	{
		BubbleToDraw++;
		if(BubbleToDraw > NUM_BUBBLES)
			BubbleToDraw = NUM_BUBBLES;

		
			for(int j = 0; j < BubbleToDraw; j++)
			{
				Bubs[nIndex][j].x += Bubs[nIndex][j].xV;//* bubTime;
				Bubs[nIndex][j].y += Bubs[nIndex][j].yV;// * bubTime;
				Bubs[nIndex][j].z += Bubs[nIndex][j].zV;// * bubTime;

 				if(Bubs[nIndex][j].y > AQHEIGHT -4)
 				{
 					Bubs[nIndex][j].x = 0;
 					Bubs[nIndex][j].y = 0;
 					Bubs[nIndex][j].z = 0;
					Bubs[nIndex][j].xV = BUBALG;
					Bubs[nIndex][j].yV = 0.2f + ((rand()%3) * 0.1f);
					Bubs[nIndex][j].zV = BUBALG;
 				}
 				if(Bubs[nIndex][j].x > GlassRadius-3)
 				{
 					Bubs[nIndex][j].x = 0;
 					Bubs[nIndex][j].y = 0;
 					Bubs[nIndex][j].z = 0;
					Bubs[nIndex][j].xV = BUBALG;
					Bubs[nIndex][j].yV = 0.2f + ((rand()%3) * 0.1f);
					Bubs[nIndex][j].zV = BUBALG;
 				}
 				if(Bubs[nIndex][j].x < -GlassRadius+3)
 				{
 					Bubs[nIndex][j].x = 0;
 					Bubs[nIndex][j].y = 0;
 					Bubs[nIndex][j].z = 0;
					Bubs[nIndex][j].xV = BUBALG;
					Bubs[nIndex][j].yV = 0.2f + ((rand()%3) * 0.1f);
					Bubs[nIndex][j].zV = BUBALG;
 				}
 				if(Bubs[nIndex][j].z > GlassRadius-3)
 				{
 					Bubs[nIndex][j].x = 0;
 					Bubs[nIndex][j].y = 0;
 					Bubs[nIndex][j].z = 0;
					Bubs[nIndex][j].xV = BUBALG;
					Bubs[nIndex][j].yV = 0.2f + ((rand()%3) * 0.1f);
					Bubs[nIndex][j].zV = BUBALG;
 				}
 				if(Bubs[nIndex][j].z < -GlassRadius+3)
 				{
 					Bubs[nIndex][j].x = 0;
 					Bubs[nIndex][j].y = 0;
 					Bubs[nIndex][j].z = 0;
					Bubs[nIndex][j].xV = BUBALG;
					Bubs[nIndex][j].yV = 0.2f + ((rand()%3) * 0.1f);
					Bubs[nIndex][j].zV = BUBALG;
 				} 

			}
		
		bubTime = timer.GetElapsedSeconds ();
	}

	glPointSize(5);
	glActiveTexture(GL_TEXTURE0);
	glBindTexture (GL_TEXTURE_2D, textureHandles[1]);
	glEnable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE1);
	glDisable (GL_TEXTURE_2D);

	glDisable(GL_LIGHTING);
	glBegin(GL_POINTS);
	{
		for(int i = 0; i < BubbleToDraw; i++)
		{
			glColor4fv(fWhite);
			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, fWhite);
			glVertex3f(Bubs[nIndex][i].x,Bubs[nIndex][i].y,Bubs[nIndex][i].z);
		}
	}
	glEnd();
	glEnable (GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE0);	
	glBindTexture (GL_TEXTURE_2D, textureHandles[0]);
	glDisable (GL_TEXTURE_2D);
	glEnable (GL_BLEND);
	glEnable(GL_LIGHTING);

}

bool LoadTextureTGA(const char *szFilename)
{
	GLbyte *pBits = NULL;
	int iHeight, iWidth;

	GLint components;
	GLenum format;
	pBits = gltLoadTGA  (szFilename,&iWidth,&iHeight,&components,&format);
	if(pBits == NULL)
	{
		MessageBox(NULL, "Hey dodo, you can't copy files", NULL, MB_OK);
		return false;
	}
	glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
	//glTexImage2D (GL_TEXTURE_2D,0,GL_RGB, iWidth,iHeight,0,GL_BGR,GL_UNSIGNED_BYTE,pBits);
	//build 2d mipmaps
	gluBuild2DMipmaps (GL_TEXTURE_2D,components,iWidth,iHeight,format,GL_UNSIGNED_BYTE,pBits);
	free(pBits); // i know the read function uses malloc

	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

	return true;
}

void ResetBubs()
{
	for(int i = 0; i < 3; i++)
	{
		for(int j = 0; j < NUM_BUBBLES; j++)
		{
			Bubs[i][j].x = 0;
			Bubs[i][j].y = 0;
			Bubs[i][j].z = 0;
		}
	}
}