// lux.fp
// Simple directional "per pixel" light shader
// Richard S. Wright Jr.

// vectors are interpolated between verticies
varying vec3 vNormal;				// Surface normal
varying vec3 vHalfVector;			// Half vector
varying vec3 vLight;				// To the light from vertex
uniform float fAlpha;

void main(void)
   {
   float pf = 0.0; // Shininess power

   // Alas, since we are interpolating these vectors, we must renormalize...
   float nDotVP= max(0.0, dot(normalize(vNormal), normalize(vLight)));
   float nDotHV = max(0.0, dot(normalize(vNormal), normalize(vHalfVector)));
   
   // Compute specular power if not zero
   if(nDotVP != 0.0)
	pf = pow(nDotHV, gl_FrontMaterial.shininess);
	
   // Add'em up and move 'em out
   vec4 vAmbientTerm = gl_FrontMaterial.ambient * gl_LightSource[0].ambient;
   vec4 vDiffuseTerm = gl_FrontMaterial.diffuse * gl_LightSource[0].diffuse * nDotVP;
   vec4 vSpecular = gl_FrontMaterial.specular * (gl_LightSource[0].specular * pf);

   // Final Fragment Color
   gl_FragColor = vAmbientTerm + vDiffuseTerm + vSpecular;
   gl_FragColor.a = fAlpha;
   }
